#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
https://github.com/wit-ai/pywit/blob/master/examples/quickstart.py

actions = {
        'send': send,
        'getForecast': get_forecast,
}
'''

from __future__ import unicode_literals

import os.path
import sys



try:
    import yige
except ImportError:
    sys.path.append(
        os.path.join(os.path.dirname(os.path.realpath(__file__)), os.pardir)
    )
    import yige

#import uuid

DEV_ACCESS_TOKEN = os.environ.get("DEV_ACCESS_TOKEN")


def main():
    ai = yige.Yige(DEV_ACCESS_TOKEN)

    # some unuque session id for user identification
    '''
    entries = [
        apiai.UserEntityEntry('Firefox', ['Firefox']), #辅助类
        apiai.UserEntityEntry('XCode', ['XCode']),
        apiai.UserEntityEntry('Sublime Text', ['Sublime Text'])
    ]
    '''
    payload= {}
    payload["name"] = "zhubo_test_from_yige_sdk"
    payload["type"] = 1
    payload["automated_expansion"] = 0
    payload["entries"] = []
    zhubo1 =  {
                "value": "逻辑思维",
                "synonyms": [
                    "罗振宇",
                    "老罗",
                    "罗胖"
                ]
            }
    payload["entries"].append(zhubo1)
    user_entities_request = ai.user_entities_request()
    user_entities_request.user_entities = payload
    user_entities_response = user_entities_request.getresponse() #真正发送
    print(user_entities_response.json())

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
try:
    from inspect import signature
except ImportError:
    from funcsigs import signature
import logging
class ActionHandle(object):

    """help to manage yige action flow
    """

    def __init__(self,query_response,logger=None):
        # module log https://fangpenlin.com/posts/2012/08/26/good-logging-practice-in-python/
        self.logger = logger or logging.getLogger(__name__)
        assert query_response.status_code == 200 #断言它是有效的query response 200
        self.query_response = query_response

    def action_a(self):
        """
        for test
        """
        return "action_a"


class MyActionHandle(ActionHandle):

    def __init__(self,query_response):
        ActionHandle.__init__(self,query_response)
    def action_c(self,**kwargs):
        '''
        可选参数
        kwargs.get("") 直接拿到key value
        '''
        return "action_c:"+str(kwargs)

def run_action(actionHandleInstance,confidence =0.5,logger=None):
        '''
        执行action
        获取所有以action开头的函数 然后拿到签名，调用它 模仿pytest
        同名函数优先级越往下越高
        conferance  默认高于0.5时才运行
        返回 {success:True,data:,message:""}
        ? 是否需要自动参数补全功能，不全的化自动要求补全  和网页一样
        '''
        #import IPython;IPython.embed()
        # todo log 输入输出都记录下
        logger = logger or logging.getLogger(__name__)
        query_response_json = actionHandleInstance.query_response.json()

        status_code = query_response_json["status"]["code"]
        answer = query_response_json["answer"]
        if status_code == 201 :
            # 不存在，直接被闲聊接手
            # todo : status 201 更精确
            response={}
            response["success"] = False
            response["answer"] = answer
            response["message"] = "chat auto by yige"
            # warning_type
            # todo 定义出一堆错误码 模仿微信
            return response
        response_confidence = query_response_json.get("confidence")
        if  response_confidence <= confidence:
            response={}
            response["success"] = False
            response["message"] = "confidence is low: {} , is should >= {}".format(query_response_json["confidence"],confidence)
            logger.warning(response["message"])
            return response
        action = query_response_json["action"]
        action_complete = action["complete"]
        if  not action_complete:
            # 强制参数是否收集完成
            response={}
            response["success"] = False
            response["answer"] = answer
            response["message"] = "action parameters do not be collect completely "
            logger.warning(response["message"])
            return response
        action_name = action["name"]# 可能不存在
        parameters_json = action["parameters"] #list to  [{}{}] 取出来然后聚合 [(a,b)(c,d)]   {} key value
        # dict([("a","b"),("c","d")])
        # 把answer加上
        parameters = dict([(p["name"],p["value"]) for p in parameters_json])
        parameters["_answer"] = answer
        action_functions_name = [function for function in dir(actionHandleInstance) if function.replace("action_","")== action_name]
        #print(action_functions_name) #none
        for action_function_name in action_functions_name:
            func = getattr(actionHandleInstance,action_function_name)
            # 根据函数的参数数量决定传参
            #fun_arg_len = len(signature(func).parameters.keys())
            #print(fun_arg_len) #0 ,1
            #return func(*[query_response][:fun_arg_len]) #动态传参 应该直接传入 action对应的参数 **kwargs
            response={}
            response["success"] = True
            response["message"] = None
            response["action_name"] = action_name #记录下命中的action
            response["answer"] = func(**parameters)
            return response
        return None

# http://stackoverflow.com/questions/4374563/python-call-all-methods-of-an-object-with-a-given-set-of-arguments




#!/usr/bin/env python
# encoding: utf-8
from yige import Yige,ActionHandle,run_action

class MyActionHandle(ActionHandle):

    def __init__(self,query_response):
        ActionHandle.__init__(self,query_response)
    def action_c(self,**kwargs):
        '''
        可选参数
        kwargs.get("") 直接拿到key value
        '''
        return "action_c:"+str(kwargs)

    def action_get_zhubo_track(self,**kwargs):
        # kwargs is dict _answer
        for name, value in kwargs.items():
            print(name,value)
        return "action ok"
    # todo 如果什么都没命中 yige会接入到闲聊


CLIENT_ACCESS_TOKEN = "E27EAB221C2FF041ABC8D1B3C40373BB" #跑鞋示例
ai = Yige(CLIENT_ACCESS_TOKEN)
request = ai.text_request()
request.query = "哈哈哈" #"我想听老罗" #"我想买鞋"
query_response = request.getresponse()
#print(query_response.json())
a_h = MyActionHandle(query_response)
action_res = run_action(a_h) #try
print(action_res)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os.path
import sys

try:
    import yige
except ImportError:
    sys.path.append(
        os.path.join(os.path.dirname(os.path.realpath(__file__)), os.pardir)
    )
    import yige

CLIENT_ACCESS_TOKEN = os.environ.get("YIGE_ACCESS_TOKEN") #export YIGE_ACCESS_TOKEN=xxx

def main():
    ai = yige.Yige(CLIENT_ACCESS_TOKEN)

    request = ai.text_request()

    #request.session_id = "<SESSION ID, UNIQUE FOR EACH USER>" # 就用微信的openid

    #request.query = "我想听老罗" #ok
    #request.query = "我想听" #ok
    # >>> python send_text_example.py 老罗
    request.query = sys.argv[1] #"老罗" #ok

    response = request.getresponse() #注意置信度 confidence
    # 各个返回值的含义：http://docs.yige.ai/Query%E6%8E%A5%E5%8F%A3.html
    if (sys.version_info > (3, 0)):
        # Python 3 code in this block
        print(response.json())
    else:
        # Python 2 code in this block
        print(repr(response.json()).decode("unicode-escape"))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
python2/3
todo:
    *  在ipython中直接调用action调试更爽
    *  命令行工具 yige --console
    *  可替换的后端 yige/zhima
'''

from __future__ import unicode_literals

try:
    #install ipython
    from IPython import embed
except:
    import pip
    pip.main(["install", "ipython"]) # install package dynamically

from prompt_toolkit import prompt
from prompt_toolkit.history import FileHistory#InMemoryHistory
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
from prompt_toolkit.contrib.completers import WordCompleter
from prompt_toolkit.styles import style_from_dict
from prompt_toolkit.token import Token
from prompt_toolkit.validation import Validator, ValidationError #验证

#from pygments.lexers import JsonLexer
import os.path
import sys
import uuid
import json

#from pprint import pprint

try:
    import yige
except ImportError:
    sys.path.append(os.path.join(
        os.path.dirname(os.path.realpath(__file__)), os.pardir))
    import yige

CLIENT_ACCESS_TOKEN = os.environ.get("YIGE_ACCESS_TOKEN"
                                     )  #export YIGE_ACCESS_TOKEN=xxx


######## prompt
DEFAULT_MAX_STEPS = 5
INTERACTIVE_PROMPT = '> '
BOTTOM_TOOLBAR_STYLE = style_from_dict({
        Token.Toolbar: '#ffffff bg:#333333',
        })

def get_bottom_toolbar_tokens(cli):
    return [(Token.Toolbar, '使用: 1.query: 我想买鞋  2.简单调试: !run xxx  3.复杂调试: !debug ')]

class myValidator(Validator):
    def validate(self, document):
        if  not  document.text: #输入为空
            raise ValidationError(message='输入不能为空',
                                  cursor_position=len(document.text))  # Move cursor to end of input.

###########


from pygments import highlight, lexers, formatters
def output_format(obj):
    '''
    python2/3
    中文没有被解析
    obj is dict
    http://stackoverflow.com/questions/35950573/python-unicode-string-to-javascript
    '''
    formatted_json = json.dumps(obj, sort_keys=True, indent=4,ensure_ascii=False).encode('utf8')
    if (sys.version_info > (3, 0)):
            # Python 3 code in this block
            colorful_json = highlight(formatted_json, lexers.JsonLexer(), formatters.TerminalFormatter())
            return colorful_json
    else:
            colorful_json = highlight(unicode(formatted_json, 'UTF-8'), lexers.JsonLexer(), formatters.TerminalFormatter())
            return colorful_json #中文没有解决




def main():
    try:
        input_function = raw_input
    except NameError:
        input_function = input  #python3
    session_id = uuid.uuid1()
    history = FileHistory('/tmp/.yige_prompt') #InMemoryHistory()
    my_completer = WordCompleter(['!print','response',"!run","!debug"])

    # yige
    ai = yige.Yige(CLIENT_ACCESS_TOKEN)
    request = ai.text_request()

    response = None
    while True:
        try:
            # lexer 从网上把词库拉下来缓存到本地，然后高亮
            # 本地输入先做好分词
            # 看文章 sqlite
            # https://github.com/eliangcs/http-prompt yige-prompt
            # https://github.com/donnemartin/haxor-news
            # 自然语言 https://github.com/jonathanslenders/python-prompt-toolkit/blob/master/examples/regular-language.py
            message = prompt(INTERACTIVE_PROMPT,
                             history=history,
                             enable_history_search=True,
                             auto_suggest=AutoSuggestFromHistory(),
                             completer=my_completer,
                             get_bottom_toolbar_tokens=get_bottom_toolbar_tokens,# 底部转态栏
                             style = BOTTOM_TOOLBAR_STYLE,
                             validator = myValidator(),
                             #get_title= fun #"Yige_prompt",
                             ).rstrip()
                             #mouse_support=True).rstrip() #支持鼠标会导致翻页问题
        except (KeyboardInterrupt, EOFError):
            return
        # 用户输入一句话，然后解析，颜色 方便调试？ console ,语法高亮
        #context = run_actions(session_id, message, context, max_steps)
        if message:
            # 仅仅是拿到输入而已，之后要做什么自己决定
            if message.startswith("!run"):
                #在当前上下文执行python
                #上次执行的结果 response 关键词 使用grep?
                code = message.split("run")[-1].strip()
                # 把response变为类似对象
                # 复杂的调试使用ipython 或者进入ipython上下文? 把当前环境带入
                # 在ipython之前诸如上下文?
                try:
                    exec(code) #在上下文执行，python提示
                except Exception as e :
                    print(str(e))
            elif message.startswith("!debug"):
                #帮助安装
                embed(header='yige.ai debug console \n    --  by 『wwj718』(blog.just4fun.site) \n    --  如有建议或bug,欢迎发我邮件:wuwenjie718@gmail.com', banner1='')

            else:
                #查询
                try:
                    request.query = message
                    response = request.getresponse()  #注意置信度 confidence
                    request.session_id = session_id
                    response = request.getresponse()
                    print(output_format(response.json()))
                except Exception as e:
                    print(str(e))


    # 各个返回值的含义：http://docs.yige.ai/Query%E6%8E%A5%E5%8F%A3.html

if __name__ == '__main__':
    main()



# encoding: utf-8
class _Serializable(object):
    """
        Abstract serializable class.
        All classes implemended this used for request parameters.
        It can be serializable to JSON values for request parameters.
    """

    def _to_dict(self):
        """
            Private method used for object serialization.
        """

        raise NotImplementedError()


class Entry(_Serializable):
    """
        User entry for class `Entity`
        Entry objects, which contain reference names and synonyms for `Entity`.
        For detail information about entries see
        http://docs.yige.ai/%E8%AF%8D%E5%BA%93%E6%8E%A5%E5%8F%A3.html#词库对象中的词库数组
    """

    @property
    def value(self):
        """
            Entry's value A canonical name to be used in place of the synonyms.
            Example: `New York`
            :rtype: str or unicode
        """

        return self._value

    @value.setter
    def value(self, value):
        """
            :type value: str or unicode
        """

        self._value = value

    @property
    def synonyms(self):
        """
            The array of synonyms.
            Example: `["New York", "@big Apple",
            "city that @{never, seldom, rarely} sleeps"]`
            :rtype: list of (str or unicode)
        """

        return self._synonyms

    @synonyms.setter
    def synonyms(self, synonyms):
        """
            :type synonyms: list of (str or unicode)
        """
        self._synonyms = synonyms

    def __init__(self, value, synonyms):
        """Construct a `Entry` and fill default values."""
        super(Entry, self).__init__()

        self._value = value
        self._synonyms = synonyms

    """Private method used for object serialization."""
    def _to_dict(self):
        return {
            'value': self.value,
            'synonyms': self.synonyms
        }


class Entity(_Serializable):
    """
        User entity for `Request`
        `Entity` is used to create, retrieve and update user-defined entity
        objects. For detail information about entities see
    """

    @property
    def name(self):
        """
            Entity name.
            :rtype: str or unicode
        """

        return self._name

    @name.setter
    def name(self, name):
        """
            :type name: str or unicode
        """

        self._name = name

    @property
    def entries(self):
        """
            Entity entries. Array of `Entry` class objects
            :rtype: list of Entry
        """

        return self._entries

    @entries.setter
    def entries(self, entries):
        """
            :type entries: list of Entry
        """

        self._entries = entries

    def __init__(self, name, entries):
        super(Entity, self).__init__()

        self.name = name
        self.entries = entries

    def _to_dict(self):
        """
            Private method used for object serialization.
        """

        return {
            'name': self.name,
            'entries': list(map(lambda x: x._to_dict(), self.entries))
        }


# encoding: utf-8
from __future__ import unicode_literals
import os
import requests


class TextRequest(object):
    '''
        doc:http://docs.yige.ai/Query%E6%8E%A5%E5%8F%A3.html
        todo:action (wit.ai)
    '''
    @property
    def query(self):
        """
            Query parameter, can be string or array of strings.
            Default equal None, nut your should fill this field before send
            request.
            :rtype: str or unicode
        """
        return self._query

    @query.setter
    def query(self, query):
        self._query = query

    def __init__(self, client_access_token, base_url, version,
                 session_id):
        # query是直接请求还是，设置，重复利用
        # 延迟触发 getresponse,使用类来做，初始化好，然后query随时调整
        self.query = None

        path = "/v1/query"
        self.query_url = base_url + path
        self.payload = {}
        self.payload["token"] = client_access_token  #客户端toke
        self.payload["session_id"] = session_id

    def getresponse(self):
        self.payload["query"] = self._query
        r = requests.post(self.query_url, data=self.payload)
        return r  #不做任何处理，直接返回

        # 如果解析出意图 or 如果没有解析出意图 (置信度)



class UserEntitiesRequest(object):
    '''
        doc: http://docs.yige.ai/%E8%AF%8D%E5%BA%93.html
        todo: CURD
    '''
    @property
    def user_entities(self):
        "user_entities parameter for specification of same user entities."
        return self._user_entities

    @user_entities.setter
    def user_entities(self, user_entities):
        self._user_entities = user_entities

    def __init__(self, client_access_token, base_url, version):#,user_entities):
        self._user_entities = None
        #self.client_access_token = client_access_token #os.environ.get("YIGE_DEV_TOKEN")  # export YIGE_DEV_TOKEN=xxx
        self.headers = {
            "Authorization": "{}".format(client_access_token),  # 开发者token
            "Content-Type": "application/json; charset=utf-8"
            #"User-Agent": "PythonClient/0.1 by python client"
        }
        path = "/v1/entities"  
        self.entity_url  = base_url + path
        #self.url = "http://httpbin.org/post"  # test 正常
    
    def getresponse(self):
            response = requests.post(self.entity_url,
                                     json=self.user_entities,
                                     headers=self.headers,
                                     verify=False)
            return response

class IntentsRequest(object):
    '''
    doc:http://docs.yige.ai/%E5%9C%BA%E6%99%AF%E6%8E%A5%E5%8F%A3.html
    todo:
        CRUD : 类比django orm
    '''
    def __init__(self, client_access_token, base_url, version):
        pass